@@ -10,11 +10,66 @@ import os, random
 from django.dispatch import receiver
 from django.db.models.signals import post_save
 from .signals import ucs_passed
-
+from PIL import Image, ExifTags
 
 # Create your models here.
 
 
+ASSIGNMENT_IMG_SIZE = (1024,768)
+COURSE_IMG_SIZE = (800,600)
+
+
+# 压缩上传作业图片
+def resize_assignment_image(img_field):
+    img = Image.open(img_field)
+
+    try:
+        for orientation in ExifTags.TAGS.keys():
+            if ExifTags.TAGS[orientation] == 'Orientation':
+                exif = dict(img._getexif().items())
+
+                if exif[orientation] == 3:
+                    img = img.rotate(180, expand=True)
+                elif exif[orientation] == 6:
+                    img = img.rotate(270, expand=True)
+                elif exif[orientation] == 8:
+                    img = img.rotate(90, expand=True)
+
+                break
+    except:
+        pass
+
+    img.thumbnail(ASSIGNMENT_IMG_SIZE, Image.ANTIALIAS)
+    img.save(img_field.path)
+
+    img.close()
+
+
+# 压缩课程课件配图
+def resize_course_image(img_field):
+    img = Image.open(img_field)
+
+    try:
+        for orientation in ExifTags.TAGS.keys():
+            if ExifTags.TAGS[orientation] == 'Orientation':
+                exif = dict(img._getexif().items())
+
+                if exif[orientation] == 3:
+                    img = img.rotate(180, expand=True)
+                elif exif[orientation] == 6:
+                    img = img.rotate(270, expand=True)
+                elif exif[orientation] == 8:
+                    img = img.rotate(90, expand=True)
+
+                break
+    except:
+        pass
+
+    img.thumbnail(COURSE_IMG_SIZE, Image.ANTIALIAS)
+    img.save(img_field.path)
+
+    img.close()
+
 
 # 课程
 class Course(models.Model):
@@ -105,6 +160,8 @@ class Section(models.Model):
         return question
 
 
+
+
 # 课件的用户笔记
 class UserSectionNote(models.Model):
     section = models.ForeignKey(Section)
@@ -159,6 +216,7 @@ class Video(models.Model):
     image = models.CharField(max_length=180, null=True)  # 视频截图地址
     # ===========
     title = models.CharField(max_length=180, null=True, verbose_name='标题')  # 视频标题
+    file_name = models.CharField(max_length=180, null=True, blank=True, verbose_name='视频文件名')
     info = models.TextField(verbose_name='说明', null=True, blank=True)
     created_at = models.DateTimeField(auto_now_add=True)
     # created_by = models.ForeignKey("users.User", related_name='cc_video_user', null=True)  # 由谁上传
@@ -207,6 +265,9 @@ class SectionQuestion(models.Model):
     class Meta:
         db_table = 'bee_django_course_section_question'
         ordering = ['order_by']
+        permissions = (
+            ('view_sectionquestion', '可以查看问题列表'),
+        )
 
     # 添加选项
     def add_options(self, count=1):
@@ -306,6 +367,7 @@ class UserLive(models.Model):
         permissions = (
             ('view_all_userlives', '可以查看所有学生的录播'),
             ('view_teach_userlives', '可以查看所教的学生的录播'),
+            ('view_child_userlives', '可以查看亲自学生的录播'),
         )
 
     def __unicode__(self):
@@ -328,6 +390,27 @@ class UserLive(models.Model):
 
         return
 
+    # 检查学员的助教是否点评了此次录播
+    def get_mentor_comments_status(self):
+        user_profile = self.user.userprofile
+        if user_profile:
+            user_class = self.user.userprofile.user_class
+            if user_class:
+                mentor = user_class.assistant
+                if mentor:
+                    mentor_comments = self.userlivecomment_set.filter(user=mentor)
+                    if mentor_comments.exists():
+                        return True
+                    else:
+                        return False
+                else:
+                    return False
+            else:
+                return False
+        else:
+            return False
+
+
 
 class UserLiveComment(models.Model):
     user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name='作者')
@@ -354,6 +437,9 @@ class UserCourse(models.Model):
 
     class Meta:
         db_table = 'bee_django_course_user_course'
+        permissions = [
+            ('assign_user_course', '能给学生分配课程'),
+        ]
 
     def get_pass(self):
         self.status = 1
@@ -381,6 +467,12 @@ class UserCourseSection(models.Model):
         permissions = [
             ('view_all_usercoursesection', '查看所有学生课件'),
             ('view_teach_usercoursessection', '查看所教的学生课件'),
+            ('view_child_usercoursesection', '查看亲子学生课件'),
+            ('pass_ucs', '能通过学生课件'),
+            ('close_ucs', '能关闭学生课件'),
+            ('open_ucs', '能开启学生课件'),
+            ('review_ucs', '能查看学生作业'),
+            ('score_ucs', '能给学生作业评分')
         ]
 
     def close(self):
@@ -391,10 +483,7 @@ class UserCourseSection(models.Model):
         self.updated_at = timezone.now()
         self.save()
 
-    def auto_pass_check(self):
-        if not self.section.auto_pass:
-            return False
-
+    def pass_check(self):
         section = self.section
         textwork_check = False
         imagework_check = False
@@ -415,7 +504,7 @@ class UserCourseSection(models.Model):
             imagework_check = True
 
         if section.has_videowork:
-            if self.work_time / 60 >= section.video_length_req:
+            if self.work_time >= section.video_length_req:
                 videowork_check = True
         else:
             videowork_check = True
@@ -432,6 +521,12 @@ class UserCourseSection(models.Model):
         else:
             return False
 
+    def auto_pass_check(self):
+        if not self.section.auto_pass:
+            return False
+
+        return self.pass_check()
+
     def open_ucs(self):
         # 如果课件为自动通过 先检查是否满足通过条件
         rc = self.auto_pass_check()
@@ -570,6 +665,7 @@ class UserAssignment(models.Model):
         self.save()
 
 
+# 删除原图，用压缩的图片代替
 class UserAssignmentImage(models.Model):
     user_course_section = models.ForeignKey(UserCourseSection, null=True, blank=True)
     image = models.ImageField(verbose_name='图片作业', upload_to='assignments/%Y/%m/%d')
@@ -579,5 +675,51 @@ class UserAssignmentImage(models.Model):
         db_table = 'bee_django_course_user_assignment_image'
 
 
+@receiver(post_save, sender=UserAssignmentImage)
+def resize_assignment_image_handler(sender, **kwargs):
+    assignment_image = kwargs['instance']
+    if kwargs['created']:
+        resize_assignment_image(assignment_image.image)
+
+
 class Preference(models.Model):
     how_to_pass = models.IntegerField(verbose_name='课程通过模式', default=0)  # 0 自动， 1 手动
+
+
+# 提醒助教和客服的默认处理方式
+from bee_django_message.exports import send_message
+from .exports import get_current_user_course, get_user_last_course_section
+
+
+def notify_mentor(self):
+    user_class = self.userprofile.user_class
+    if not user_class:
+        return -1
+    if not user_class.assistant:
+        return -2
+
+    mentor = self.userprofile.user_class.assistant
+    ucs = get_user_last_course_section(self)
+    title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
+
+    uc = get_current_user_course(self)
+    url = '/course/user_course_detail/' + str(uc.id)
+
+    send_message(to_user=mentor, title=title, url=url)
+
+    return 0
+
+def notify_agent(self):
+    uc = get_current_user_course(self)
+    ucs = get_user_last_course_section(self)
+    url = '/course/user_course_detail/' + str(uc.id)
+
+    title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
+
+    send_message(from_user=self, title=title, url=url, message_identity='course_notify')
+
+    return 0
+
+
+User.add_to_class('notify_mentor', notify_mentor)
+User.add_to_class('notify_agent', notify_agent)
