@@ -51,18 +51,27 @@ urlpatterns = [
     url(r'^course/section/question/list_json/(?P<section_id>[0-9]+)/$', views.SectionQuestionListJson.as_view(),
         name='section_question_list_json'),
     url(r'^course/section/question/add/$', views.SectionQuestionCreate.as_view(), name='section_question_add'),
-    url(r'^course/section/question/save/(?P<question_id>[0-9]+)/$', views.SectionQuestionSave.as_view(), name='save_section_question'),
-    url(r'^course/section/question/delete/(?P<question_id>[0-9]+)/$', views.SectionQuestionDelete.as_view(), name='section_question_delete'),
-    url(r'^course/section/option/add/(?P<question_id>[0-9]+)/$', views.SectionQuestionOptionCreate.as_view(), name='question_option_add'),
-    url(r'^course/section/option/delete/(?P<option_id>[0-9]+)/$', views.SectionQuestionOptionDelete.as_view(), name='question_option_delete'),
-    url(r'^course/section/question/answer/(?P<user_section_id>[0-9]+)/$', views.SectionQuestionAnswer.as_view(), name='question_answer'),
-    url(r'^user/question/answer/record/(?P<user_section_id>[0-9]+)/$', views.UserQuestionAnswerRecordList.as_view(), name='user_question_answer'),
+    url(r'^course/section/question/save/(?P<question_id>[0-9]+)/$', views.SectionQuestionSave.as_view(),
+        name='save_section_question'),
+    url(r'^course/section/question/delete/(?P<question_id>[0-9]+)/$', views.SectionQuestionDelete.as_view(),
+        name='section_question_delete'),
+    url(r'^course/section/option/add/(?P<question_id>[0-9]+)/$', views.SectionQuestionOptionCreate.as_view(),
+        name='question_option_add'),
+    url(r'^course/section/option/delete/(?P<option_id>[0-9]+)/$', views.SectionQuestionOptionDelete.as_view(),
+        name='question_option_delete'),
+    url(r'^course/section/question/answer/(?P<user_section_id>[0-9]+)/$', views.SectionQuestionAnswer.as_view(),
+        name='question_answer'),
+    url(r'^user/question/answer/record/(?P<user_section_id>[0-9]+)/$', views.UserQuestionAnswerRecordList.as_view(),
+        name='user_question_answer'),
+    url(r'^user/question/answer/record/detail/(?P<pk>[0-9]+)/$', views.UserQuestionAnswerRecordDetail.as_view(),
+        name='user_question_answer_detail'),
 
     # ===== 视频 ======
     url(r'^video/list/$', views.VideoList.as_view(), name='video_list'),
     url(r'^video/detail/(?P<pk>[0-9]+)/$', views.VideoDetail.as_view(), name='video_detail'),
     url(r'^video/update/(?P<pk>[0-9]+)/$', views.VideoUpdate.as_view(), name='video_update'),
     url(r'^video/upload/$', views.VideoUpload.as_view(), name='video_upload'),
+    url(r'^video/upload_to_qiniu$', views.VideoUploadToQiniu.as_view(), name='video_upload_to_qiniu'),
     # 1.选取文件后,获取video信息
     url(r'^cc/video/info/add/$', views.CCVideoInfoCreate.as_view(), name='cc_video_info_add'),
     # 2.上传完成后，记录video信息到数据库
@@ -71,6 +80,10 @@ urlpatterns = [
     url(r'^cc/video/callback/', views.cc_video_callback, name='cc_video_callback'),
     # url(r'^vodio/play/$', views.play_video, name='play_video'),
 
+    # 获取七牛云上传token
+    url(r'^uptoken$', views.uptoken, name='uptoken'),
+    url(r'^add_qiniu_video_to_video$', views.add_qiniu_video_to_video, name='add_qiniu_video_to_video'),
+
     # ===== 录播 ======
     url(r'^live/list/(?P<user_id>[0-9]+)/$', views.LiveList.as_view(), name='live_list'),
     # url(r'^live/list/(?P<user_id>[0-9]+)/$', views.UserLiveList.as_view(), name='user_live_list'),
@@ -78,6 +91,12 @@ urlpatterns = [
     url(r'^live/video/(?P<live_video_id>(.)+)/$', views.live_video_detail, name='live_video_detail'),
     # 用户查看录播视频
     url(r'^user/live/(?P<live_video_id>(.)+)/$', views.user_live_video_detail, name='user_live_video_detail'),
+    # 编辑录播评论
+    url(r'^user_live/comment/(?P<user_live_comment_id>\d+)/edit$', views.edit_user_live_comment,
+        name='edit_user_live_comment'),
+    # 删除录播评论
+    url(r'^user_live/comment/(?P<user_live_comment_id>\d+)/delete$', views.delete_user_live_comment,
+        name='delete_user_live_comment'),
     # 直播/录播 结束回调 2
     url(r'^live_end_callback$', views.cc_live_end_callback, name='user_live_end_callback'),
     # 直播 完成回调 103
@@ -111,7 +130,10 @@ urlpatterns = [
     url(r'^delete_user_course/(?P<user_course_id>\d+)$', views.delete_user_course, name='delete_user_course'),
     # 查看用户待评分作业列表
     url(r'^manage_assignments$', views.manage_user_assignments, name='manage_user_assignments'),
-
+    # 提醒助教
+    url(r'^user_(?P<user_id>\d+)/notify_mentor$', views.notify_mentor, name='notify_mentor'),
+    # 申请客服
+    url(r'^user_(?P<user_id>\d+)/notify_agent$', views.notify_agent, name='notify_agent'),
     # 查看指定用户所有作业
     url(r'^manage/user/(?P<user_id>\d+)/assignments$', views.manage_user_assignment_list,
         name='manage_user_assignment_list'),
@@ -147,4 +169,11 @@ urlpatterns = [
     # 用户通过答题状态记录
     url(r'^ucs_question_passed/(?P<ucs_id>\d+)$',
         views.ucs_question_passed, name='ucs_question_passed'),
+
+    # 微信小程序接口
+    url(r'weixin_user_course_index$', views.weixin_user_course_index, name='weixin_user_course_index'),
+    url(r'weixin_ucs_detail/(?P<ucs_id>\d+)$', views.weixin_ucs_detail, name='weixin_ucs_detail'),
+
+    url(r'test_video$', views.test_video, name='test_video'),
+    url(r'uptoken$', views.uptoken, name='uptoken'),
 ]
