@@ -16,14 +16,15 @@ from django.contrib import messages
 from django.contrib.auth import get_user_model
 from django.conf import settings
 from .models import Course, Section, CourseSectionMid, Video, UserLive, UserCourse, UserCourseSection, \
-    Preference, UserSectionNote, SectionVideo, SectionQuestion, SectionQuestionOption, UserQuestionAnswerRecord
+    Preference, UserSectionNote, SectionVideo, SectionQuestion, SectionQuestionOption, UserQuestionAnswerRecord, \
+    UserLiveComment
 from .forms import CourseForm, SectionForm, CourseSectionForm, CourseSectionOrderForm, VideoForm, UploadImageForm, \
     UserCourseForm, UserAssignmentForm, UserAssignmentImageForm, SectionVideoForm, PreferenceForm, \
     section_attach_form, UserSectionNoteForm, SectionVideoOrderForm, LivePlayLogsForm, UserLiveCommentForm, \
     CourseSectionQuestionSearchForm
 from .utils import JSONResponse, page_it
 from .cc import create_video_info, get_video_str, get_play_logs
-from django.http import JsonResponse
+from django.http import JsonResponse, HttpResponseRedirect
 from django.views.decorators.csrf import csrf_exempt
 from django.db import transaction
 from django.utils import timezone
@@ -31,6 +32,11 @@ from bee_django_course import signals
 from django.apps import apps
 from .exports import get_teach_users, get_user_course_section_list
 import cc
+from dss.Serializer import serializer
+from django.core.exceptions import PermissionDenied
+
+from qiniu import Auth
+from qiniu import BucketManager
 
 User = get_user_model()
 
@@ -50,13 +56,15 @@ class CourseRedirectView(RedirectView):
 
     def get_redirect_url(self, *args, **kwargs):
         if self.request.user.has_perm('bee_django_course.view_all_courses'):
-            # 跳转 CourseList
+            # 跳转课程列表 CourseList
             self.pattern_name = 'bee_django_course:course_list'
-        elif self.request.user.has_perm('bee_django_course.view_teach_usercoursessection'):
-            # self.pattern_name = 'bee_django_course:live_list'
+        if self.request.user.has_perm('bee_django_course.view_teach_usercoursessection') \
+                or self.request.user.has_perm('bee_django_course.view_all_usercoursesection') \
+                or self.request.user.has_perm('bee_django_course.view_child_usercoursesection'):
+            # 跳转录播列表
             self.url = reverse('bee_django_course:live_list', kwargs={"user_id": 0})
         else:
-            self.url = '/'
+            raise PermissionDenied
         return super(CourseRedirectView, self).get_redirect_url(*args, **kwargs)
 
 
@@ -74,6 +82,8 @@ class CourseList(ListView):
             return super(CourseList, self).get(request, *args, **kwargs)
         elif request.user.has_perm('bee_django_course.view_teach_usercoursessection'):
             return redirect(reverse('bee_django_course:live_list', kwargs={"user_id": 0}))
+        else:
+            raise PermissionDenied
 
 
 @method_decorator(cls_decorator(cls_name='CourseDetail'), name='dispatch')
@@ -121,6 +131,12 @@ class SectionList(ListView):
     paginate_by = 20
     queryset = Section.objects.all()
 
+    def get(self, request, *args, **kwargs):
+        if request.user.has_perm("bee_django_course.view_all_sections"):
+            return super(SectionList, self).get(request, *args, **kwargs)
+        else:
+            raise PermissionDenied
+
     def get_queryset(self):
         section_name = self.request.GET.get('section_name')
         if section_name:
@@ -303,13 +319,26 @@ class VideoList(ListView):
     template_name = 'bee_django_course/video/video_list.html'
     context_object_name = 'video_list'
     paginate_by = 20
-    queryset = Video.objects.all()
+
+    def get(self, request, *args, **kwargs):
+        if request.user.has_perm("bee_django_course.view_all_videos"):
+            return super(VideoList, self).get(request, *args, **kwargs)
+        else:
+            raise PermissionDenied
 
     def get_context_data(self, **kwargs):
         context = super(VideoList, self).get_context_data(**kwargs)
         context["video_provider_name"] = settings.COURSE_VIDEO_PROVIDER_NAME
         return context
 
+    def get_queryset(self):
+        video_name_params = self.request.GET.get('video_name')
+        if video_name_params:
+            queryset = Video.objects.filter(title__contains=video_name_params)
+        else:
+            queryset = Video.objects.all()
+        return queryset
+
 
 @method_decorator(cls_decorator(cls_name='VideoDetail'), name='dispatch')
 class VideoDetail(DetailView):
@@ -323,6 +352,8 @@ class VideoDetail(DetailView):
         video_src = None
         if settings.COURSE_VIDEO_PROVIDER_NAME == 'cc':
             video_src = get_video_str(video.video_id)
+        elif settings.COURSE_VIDEO_PROVIDER_NAME == 'qiniu':
+            video_src = "http://" + settings.QINIU_DOMAIN + "/" + video.file_name
         context["video_src"] = video_src
         context["video_provider_name"] = settings.COURSE_VIDEO_PROVIDER_NAME
         return context
@@ -357,26 +388,113 @@ def prepare_live_video_page_data(live_video_id):
 
 def live_video_detail(request, live_video_id):
     user_live = UserLive.objects.get(record_video_id=live_video_id)
+    comments = user_live.userlivecomment_set.order_by('-submit_date')
+
+    if request.method == "POST":
+        comment_form = UserLiveCommentForm(data=request.POST)
+        if comment_form.is_valid():
+            comment = comment_form.save(commit=False)
+            comment.user = request.user
+            comment.user_live = user_live
+            comment.save()
+            signals.user_live_comment.send(sender=UserLiveComment, user_live=user_live, comment_user=request.user)
+            return redirect(reverse('bee_django_course:live_video_detail', kwargs={'live_video_id': live_video_id}))
+    else:
+        comment_form = UserLiveCommentForm()
+
     return render(request, 'bee_django_course/live/live_video_detail.html', context={
         'record_video_id': live_video_id,
-        'user_live': user_live
+        'user_live': user_live,
+        'comments': comments,
+        'comment_form': comment_form,
     })
 
 
 def user_live_video_detail(request, live_video_id):
     user_live = UserLive.objects.get(record_video_id=live_video_id)
+    comments = user_live.userlivecomment_set.order_by('-submit_date')
+
+    if request.method == "POST":
+        comment_form = UserLiveCommentForm(data=request.POST)
+        if comment_form.is_valid():
+            comment = comment_form.save(commit=False)
+            comment.user = request.user
+            comment.user_live = user_live
+            comment.save()
+            signals.user_live_comment.send(sender=UserLiveComment, user_live=user_live, comment_user=request.user)
+            return redirect(
+                reverse('bee_django_course:user_live_video_detail', kwargs={'live_video_id': live_video_id}))
+    else:
+        comment_form = UserLiveCommentForm()
+
     return render(request, 'bee_django_course/user/live_video_detail.html', context={
         'record_video_id': live_video_id,
-        'user_live': user_live
+        'user_live': user_live,
+        'comments': comments,
+        'comment_form': comment_form,
     })
 
 
+def edit_user_live_comment(request, user_live_comment_id):
+    comment = get_object_or_404(UserLiveComment, pk=user_live_comment_id)
+
+    if request.method == "POST":
+        next = reverse('bee_django_course:user_live_video_detail',
+                       kwargs={'live_video_id': comment.user_live.record_video_id})
+        comment_form = UserLiveCommentForm(data=request.POST, instance=comment)
+        if comment_form.is_valid():
+            comment_form.save()
+            return HttpResponseRedirect(next)
+    else:
+        comment_form = UserLiveCommentForm(instance=comment)
+
+    return render(request, 'bee_django_course/live/edit_user_live_comment.html', context={
+        'comment_form': comment_form,
+    })
+
+
+def delete_user_live_comment(request, user_live_comment_id):
+    comment = get_object_or_404(UserLiveComment, pk=user_live_comment_id)
+
+    if request.method == "POST":
+        next = reverse('bee_django_course:user_live_video_detail',
+                       kwargs={'live_video_id': comment.user_live.record_video_id})
+        comment.delete()
+        return JsonResponse(data={
+            'message': '操作成功',
+            'next': next,
+        })
+
+
 @method_decorator(cls_decorator(cls_name='VideoUpload'), name='dispatch')
 class VideoUpload(TemplateView):
     def get(self, request, *args, **kwargs):
         return render(request, 'bee_django_course/video/video_upload.html')
 
 
+@method_decorator(cls_decorator(cls_name='VideoUpload'), name='dispatch')
+class VideoUploadToQiniu(TemplateView):
+    def get(self, request, *args, **kwargs):
+        return render(request, 'bee_django_course/video/video_upload_to_qiniu.html')
+
+
+# 将七牛云的视频，添加至Video
+def add_qiniu_video_to_video(request):
+    if request.method == "POST":
+        file_name = request.POST.get('file_name')
+        try:
+            video = Video.objects.create(title=file_name, file_name=file_name)
+            return JsonResponse(data={
+                'rc': 0,
+                'message': '创建成功'
+            })
+        except:
+            return JsonResponse(data={
+                'rc': -1,
+                'message': '添加失败'
+            })
+
+
 # ======上传视频=====
 # 1 创建点播信息
 @method_decorator(cls_decorator(cls_name='CCVideoInfoCreate'), name='dispatch')
@@ -415,6 +533,58 @@ def cc_video_callback(request):
     return
 
 
+# 上传视频到七牛，需要先获取的token
+def get_qiniu_token(key):
+    access_key = settings.QINIU_AK
+    secret_key = settings.QINIU_SK
+    # 构建鉴权对象
+    q = Auth(access_key, secret_key)
+    # 要上传的空间
+    bucket_name = settings.QINIU_BUCKET_NAME
+    # key 上传后保存的文件名
+
+    # 生成上传 Token，可以指定过期时间等
+    # 上传策略示例
+    # https://developer.qiniu.com/kodo/manual/1206/put-policy
+    policy = {
+        # 'callbackUrl':'https://requestb.in/1c7q2d31',
+        # 'callbackBody':'filename=$(fname)&filesize=$(fsize)'
+        # 'persistentOps':'imageView2/1/w/200/h/200'
+    }
+    # 3600为token过期时间，秒为单位。3600等于一小时
+    token = q.upload_token(bucket_name, key=key.encode('utf-8'), expires=3600, policy=policy)
+    return token
+
+
+def uptoken(request):
+    key = request.GET.get('key')
+    token = get_qiniu_token(key)
+    return JsonResponse(data={
+        'uptoken': token,
+        'domain': settings.QINIU_DOMAIN,
+    })
+
+
+def add_video_to_part(request, part_id):
+    if request.method == "POST":
+        file_name = request.POST.get('file_name')
+        try:
+            part = get_object_or_404(Part, pk=part_id)
+            video_count = part.video_set.all().count() + 1
+            part.video_set.create(file_name=file_name, number=video_count)
+
+            return JsonResponse(data={
+                'rc': 0,
+                'message': '创建成功'
+            })
+        except Part.DoesNotExist:
+            return JsonResponse(data={
+                'rc': -1,
+                'message': '未找到对应小节'
+            })
+
+
+
 # 查看所有的学生录播
 @method_decorator(cls_decorator(cls_name='LiveList'), name='dispatch')
 class LiveList(ListView):
@@ -432,12 +602,23 @@ class LiveList(ListView):
 
     def get_queryset(self):
         if self.request.user.has_perm("bee_django_course.view_all_userlives"):
-            self.queryset = UserLive.objects.filter(status=1)
-        elif self.request.user.has_perm("bee_django_course.view_teach_userlives"):
-            users = get_teach_users(self.request.user)
-            self.queryset = UserLive.objects.filter(user__in=users, status=1).order_by('-created_at')
+            q1 = Q(status=1)
         else:
-            return []
+            q1 = Q(pk=None)
+
+        if self.request.user.has_perm("bee_django_course.view_teach_userlives"):
+            users = get_teach_users(self.request.user)
+            q2 = Q(user__in=users, status=1)
+        else:
+            q2 = Q(pk=None)
+
+        if self.request.user.has_perm('bee_django_course.view_child_userlives'):
+            users = self.request.user.get_child_users()
+            q3 = Q(user__in=users, status=1)
+        else:
+            q3 = Q(pk=None)
+
+        self.queryset = UserLive.objects.filter(q1|q2|q3).distinct()
         user = self.get_user()
         if user:
             self.queryset = self.queryset.filter(user=user)
@@ -628,6 +809,7 @@ def manage_user_course_section_list(request, user_course_id):
 
 
 # 查看用户所学课程的课件 依据课件作业方式的设置，添加不同的提交作业按钮
+# !!! 似乎弃用了
 def user_course_section_detail(request, ucs_id):
     ucs = get_object_or_404(UserCourseSection, pk=ucs_id)
     section = ucs.section
@@ -650,7 +832,8 @@ def user_course_section_detail(request, ucs_id):
 
     all_notes_list = section.usersectionnote_set.filter(is_open=True).order_by('-created_at')
     all_notes = page_it(request, query_set=all_notes_list)
-    my_notes = section.usersectionnote_set.filter(user=request.user).order_by('-created_at')
+    my_notes_list = section.usersectionnote_set.filter(user=request.user).order_by('-created_at')
+    my_notes = page_it(request, query_set=my_notes_list)
 
     return render(request, 'bee_django_course/user/user_course_section_detail.html', context={
         'ucs': ucs,
@@ -677,9 +860,10 @@ class UserCourseSectionDetail(DetailView):
         note_form = UserSectionNoteForm()
         context["all_notes"] = page_it(self.request, query_set=all_notes)
         context["open_notes"] = open_notes
-        context["my_notes"] = my_notes
+        context["my_notes"] = page_it(self.request, query_set=my_notes)
         context["note_form"] = note_form
         context["questions"] = section.get_questions()
+        context['video_provider_name'] = settings.COURSE_VIDEO_PROVIDER_NAME
         return context
 
     def post(self, request, *args, **kwargs):
@@ -803,6 +987,10 @@ def user_assignment_image_upload(request, ucs_id):
         form = UserAssignmentImageForm(request.POST, request.FILES)
 
         if form.is_valid():
+            if not form.cleaned_data['image']:
+                messages.error(request, '请先选择要上传的图片')
+                return redirect(reverse('bee_django_course:user_assignment', kwargs={'ucs_id': ucs.id}))
+
             assignment = form.save(commit=False)
             assignment.user_course_section = ucs
             assignment.save()
@@ -817,7 +1005,10 @@ def user_assignment_image_upload(request, ucs_id):
             messages.success(request, '上传成功')
             return redirect(reverse('bee_django_course:user_assignment', kwargs={'ucs_id': ucs.id}))
         else:
-            print form.errors
+            return render(request, template_name="bee_django_course/user/user_assignment.html", context={
+                'ucs': ucs,
+                'image_form': form,
+            })
 
 
 # 用户查看课程
@@ -848,9 +1039,9 @@ def choose_user_course(request, user_id):
             # 自动开启第一课
             setion_list = get_user_course_section_list(user_course)
             first_section = setion_list.first()
-            first_section.open_ucs()
-
-            return redirect(reverse('bee_django_course:choose_user_course', kwargs={'user_id': user.id}))
+            if first_section:
+                first_section.open_ucs()
+                return redirect(reverse('bee_django_course:choose_user_course', kwargs={'user_id': user.id}))
         else:
             messages.error(request, '表单错误')
 
@@ -892,18 +1083,33 @@ def manage_user_course(request, user_id):
 
 # 管理查看用户待评分文字作业
 def manage_user_assignments(request):
+    student_name = request.GET.get('student_name')
+
     if request.user.has_perm("bee_django_course.view_all_usercoursesection"):
-        data_list = UserCourseSection.objects.filter(status__in=[1, 4]) \
-            .filter(Q(userassignment__isnull=False) | Q(userassignmentimage__isnull=False)) \
-            .order_by('-updated_at').distinct()
-    elif request.user.has_perm("bee_django_course.view_teach_usercoursessection"):
-        users = get_teach_users(request.user)
-        data_list = UserCourseSection.objects.filter(status__in=[1, 4]) \
-            .filter(user_course__user__in=users) \
-            .filter(Q(userassignment__isnull=False) | Q(userassignmentimage__isnull=False)) \
-            .order_by('-updated_at').distinct()
+        q1 = Q()
     else:
-        data_list = []
+        q1 = Q(pk=None)
+
+    if request.user.has_perm("bee_django_course.view_teach_usercoursessection"):
+        users = get_teach_users(request.user)
+        q2 = Q(user_course__user__in=users)
+    else:
+        q2 = Q(pk=None)
+
+    if request.user.has_perm("bee_django_course.view_child_usercoursesection"):
+        users = request.user.get_child_users()
+        q3 = Q(user_course__user__in=users)
+    else:
+        q3 = Q(pk=None)
+
+
+    data_list = UserCourseSection.objects.filter(status__in=[1, 4])\
+        .filter(Q(userassignment__isnull=False) | Q(userassignmentimage__isnull=False))\
+        .filter(q1|q2|q3).distinct().order_by('-updated_at')
+
+    if student_name:
+        data_list = data_list.filter(Q(user_course__user__first_name__contains=student_name)
+                                     | Q(user_course__user__username=student_name))
 
     data = page_it(request, query_set=data_list)
 
@@ -1014,8 +1220,61 @@ def close_user_course_section(request, ucs_id):
         })
 
 
+# 手动通过时，用户提醒助教
+def notify_mentor(request, user_id):
+    try:
+        user = User.objects.get(pk=user_id)
+        rc = user.notify_mentor()
+
+        if rc == 0:
+            return JsonResponse(data={
+                'rc': 0,
+                'msg': '提醒成功'
+            })
+        elif rc == -1:
+            return JsonResponse(data={
+                'rc': -1,
+                'msg': '用户还没有班级'
+            })
+        elif rc == -2:
+            return JsonResponse(data={
+                'rc': -1,
+                'msg': '用户班级未分配助教'
+            })
+        else:
+            return JsonResponse(data={
+                'rc': -1,
+                'msg': '未知错误'
+            })
+    except User.DoesNotExist:
+        return JsonResponse(data={
+            'rc': -1,
+            'msg': '用户不存在'
+        })
+
+
+# 手动通过时，用户申请客服
+def notify_agent(request, user_id):
+    try:
+        user = User.objects.get(pk=user_id)
+        user.notify_agent()
+
+        return JsonResponse(data={
+            'rc': 0,
+            'msg': '提醒成功'
+        })
+    except User.DoesNotExist:
+        return JsonResponse(data={
+            'rc': -1,
+            'msg': '用户不存在'
+        })
+
+
 def user_list(request):
     # user_list = User.objects.filter(is_active=True).order_by('date_joined')
+    if not request.user.has_perm('bee_django_course.assign_user_course'):
+        raise PermissionDenied
+
     user_list = User.objects.order_by('date_joined')
     users = page_it(request, query_set=user_list)
 
@@ -1026,9 +1285,12 @@ def user_list(request):
 
 def save_cc_live(userId, roomId, liveId, stopStatus, recordStatus, recordVideoId, recordVideoDuration, replayUrl,
                  startTime, endTime):
-    try:
-        cclive = UserLive.objects.get(live_id=liveId)
-    except:
+    cclives = UserLive.objects.filter(live_id=liveId)
+    if cclives.exists():
+        cclive = cclives.first()
+        if cclives.count() > 1:
+            cclives.exclude(pk=cclive.id).delete()
+    else:
         cclive = UserLive()
         cclive.status = 1
         cclive.stop_status = stopStatus
@@ -1475,6 +1737,47 @@ class UserQuestionAnswerRecordList(ListView):
             return super(UserQuestionAnswerRecordList, self).get(request, *args, **kwargs)
 
 
+class UserQuestionAnswerRecordDetail(DetailView):
+    model = SectionQuestion
+    template_name = 'bee_django_course/question/question_reocrd_detail.html'
+    context_object_name = 'question'
+
+    def get_context_data(self, **kwargs):
+        context = super(UserQuestionAnswerRecordDetail, self).get_context_data(**kwargs)
+        question_id = self.kwargs["pk"]
+        question = SectionQuestion.objects.get(id=question_id)
+        option_list = question.options.all()
+        res_queryset = UserQuestionAnswerRecord.objects.filter(question_id=question_id).values('answer',
+                                                                                               'answer__option').annotate(
+            answer_count=Count('answer')).order_by().order_by("-answer_count")
+
+        res_list = []
+        user_count = 0
+        for o in option_list:
+            d = {}
+            found = False
+            for i in res_queryset:
+                if o.id == i["answer"]:
+                    d["key"] = o.option
+                    d["value"] = i["answer_count"]
+                    user_count += i["answer_count"]
+                    found = True
+                    res_list.append(d)
+            if found == False:
+                d["key"] = o.option
+                d["value"] = 0
+                res_list.append(d)
+
+        # for res_dict in res_queryset:
+        #     # option_id = res_dict["answer"]
+        #     count = res_dict["answer_count"]
+
+        context["user_count"] = user_count
+        context["res_list"] = res_list
+        # context["res_queryset"]=res_queryset
+        return context
+
+
 # 用户答题正确，更新ucs状态，返回用户学习课程的列表
 def ucs_question_passed(request, ucs_id):
     if request.method == "POST":
@@ -1531,3 +1834,52 @@ def delete_user_live(request, user_live_id):
         return JsonResponse(data={
             'rc': 0,
         })
+
+
+# 微信小程序接口
+# 课程页
+def weixin_user_course_index(request):
+    user = request.token_user
+    if not user:
+        return JsonResponse(data={
+            'msg': '无效的用户',
+        })
+
+    ucs = get_user_last_course_section(user)
+    if not ucs:
+        return JsonResponse(data={
+            'msg': '没有学习中的课程',
+        })
+
+    return JsonResponse(data={
+        'user_name': user.first_name or user.username,
+        'section_detail': serializer(ucs.section, output_type='json',
+                                     datetime_format='string', exclude_attr=['image', 'info']),
+        'ucs_detail': serializer(ucs, output_type='json', datetime_format='string'),
+        'msg': 'OK',
+    })
+
+
+# 课程详情页
+def weixin_ucs_detail(request, ucs_id):
+    try:
+        ucs = UserCourseSection.objects.get(pk=ucs_id)
+        videos = []
+        for e in ucs.section.sectionvideo_set.all():
+            videos.append({'vid': e.video.video_id, 'titile': e.video.title})
+
+        return JsonResponse(data={
+            'rc': 0,
+            'section_detail': serializer(ucs.section, output_type='json', datetime_format='string',
+                                         exclude_attr=['image']),
+            'section_videos': serializer(videos, output_type='json')
+        })
+    except UserCourseSection.DoesNotExist:
+        return JsonResponse(data={
+            'rc': -1,
+            'msg': '用户课程(UCS)不存在',
+        })
+
+
+def test_video(request):
+    return render(request, 'bee_django_course/test_video.html')
